# Author: Murat Uenalan (muenalan@cpan.org)
#
# Copyright (c) 2001 Murat Uenalan. All rights reserved.
#
# Note: This program is free software; you can redistribute
#
# it and/or modify it under the same terms as Perl itself.

1;
__END__

=head1  CLASS

SQL::Generator::Argument

=head1 SYNOPSIS

use SQL::Statement::Argument;

=head1 DESCRIPTION

SQL::Generator::Argument is a class for constructing elements of sql-statements.
It is not to be used for standalone ,more for helping SQL::Generator::Argument.
sql-statements.

=cut

=head2 EXPORT

None by default.

=head1 AUTHOR

Murat Uenalan (muenalan@cpan.org)

=head1 COPYRIGHT

Copyright (c) 1998-2002 Murat Uenalan. Germany. All rights reserved.

You may distribute under the terms of either the GNU General Public
License or the Artistic License, as specified in the Perl README file.

=head1 SEE ALSO

perl(1), SQL::Generator, SQL::Generator::Command, SQL::Generator::Lang::*

=cut
